# Simulations for larval development times in Spodoptera frugiperda
# based on Andre Gergs, Christian Baden. 2020. AmP Spodoptera frugiperda, version 2020/07/21

# implementation by Andre Gergs 2020-08-06


#simulation settings--------------------------------------------------------------------------------
tmax    <-  100                # simulation time [days]
MC_max <-100 #repetitions (number of different individuals)

Temp      <-rep(c(18), tmax ) 
#Temp [6:12]<-22

#--scenario 1------
  # Eier 1.6. bis 4.6. bei 24C	L1
  # Fr  9.6.	4.6. bis 8.6. bei 20C 	L2 mitte	100
  #           8.6. bis 9.6. bei 18C	L2 ende	100

  # Temp      <-rep(c(24), tmax )                      
  # Temp [4:7]<-20
  # Temp [8:9]<-18

#--scenario 2------
  # Eier	    8.6. bis 11.6. bei 24C	L1	100
  # Fr       15.6.	11.6. - 15.6. bei 22C	L2 ende	100
  
  # Temp      <-rep(c(24), tmax )
  # Temp [4:7]<-22

#--scenario 3------
  # Eier	    12.6. bis 15.6. bei 24C	L1 Ende	100
  # Fr 19.6.	15.6. bis 16.6. bei 18C	L1	100
  #           16.6. bis 17.6. bei 20C	L1	100
  #           17.6. bis 18.6. bei 24C	L1-L2Anfang	 20 : 80
  #           18.6. bis 19.6. bei 30C	L2 ende	100

   # temp <-rep(c(24), tmax )
   # temp [4]<-18
   # temp [5]<-20
   # temp [6]<-24
   # temp [7]<-30

##--matrices---------------------------------------------------------------------------------------
res <- matrix(data = NA, nrow = tmax, ncol = 4) # results matrix growth
res_times <- matrix(data = NA, nrow = MC_max, ncol = 7) # results matrix development times including total time
res_times <- matrix(data = NA, nrow = MC_max, ncol = 6) # results matrix development times

# DEB parameters-----------------------------------------------------------------------------------
kap = 0.7827
v = 0.009333
p_M = 51.74
E_G = 4400

s_1 	<- 	3.141 	  # 	stress at instar 1: L_1^2/ L_b^2
s_2 	<- 	2.266 	  # 	stress at instar 2: L_2^2/ L_1^2
s_3 	<- 	2.293 	  # 	stress at instar 3: L_3^2/ L_2^2
s_4 	<- 	2.671 	  # 	stress at instar 4: L_4^2/ L_3^2
s_5 	<- 	2.889 	  # 	stress at instar 5: L_5^2/ L_4^2

T_A 	<- 	8320 	    # 	Arrhenius temperature
T_AL 	<- 	1.78e+04 	# 	lower Arrhenius temperature
T_L 	<- 	289.4 	  # 	lower boundary for Arrhenius temperature
T_A1 	<- 	6668 	    # 	Arrhenius temperature for 1st instar
#T_AH <- 	3e+05 	  # 	upper Arrhenius temperature
#T_H 	<- 	304.6 	  # 	High Boundary Arrhenius temperature
#T_As <- 	1.261e+04 # 	Arrhenius temperature for larval mortality

del_M = 0.354
#set.seed(42)
#rand <- rnorm(MC_max, mean=1, sd=0.084)
rand <- rnorm(MC_max, mean=1, sd=0.087)

for (m in 1:MC_max ) {
    
     #parameter variability
      pAm <- 39.6495 * rand[m]
      E_m     <- pAm/v
      g       <- E_G/ (kap*E_m)   # energy investment ratio          
      km      <- p_M/E_G   # somatic maintencance rate coefficent
      L_b     <- 0.0109512    # structural length at birth
      Lm      <-v/(g*km)
      
    #food level
      f<-1
      #f<-0.8 #
      # f<-0.5
     # cm, head length of instar 1-6
      L_1 = L_b 
      L_2 = L_1 * s_1^0.5 
      L_3 = L_2 * s_2^0.5 
      L_4 = L_3 * s_3^0.5 
      L_5 = L_4 * s_4^0.5 
      L_6 = L_5 * s_5^0.5 
      L_e = 0.239841
      
      L_1set =FALSE
      L_2set =FALSE
      L_3set =FALSE
      L_4set =FALSE
      L_5set =FALSE
      L_6set =FALSE

       #intial state variables of individuals in vectors
        L<-L_b
        iter=100
        
        t=0
        #time loop---------------------------------------------------------------------------------------------
        #for (t in 1:tmax ) {
          
          while ((L_6set == FALSE) & (t<tmax)) {
            
            t<-t+1
                 # temperature correction
                 if (Temp[t]+273.15 > T_L) {
                     if (L < L_2) TC <- exp((T_A1/(20+273.15)-T_A1/(Temp[t]+273.15)))
                     else TC <- exp((T_A/(20+273.15)-T_A/(Temp[t]+273.15)))
                  } else TC <- exp((T_AL/(20+273.15)-T_AL/(Temp[t]+273.15)))
                
                  # if (Temp[t]+273.15 > T_L) TC <- exp((T_A/(20+273.15)-T_A/(Temp[t]+273.15)))
                   # else TC <- exp((T_AL/(20+273.15)-T_AL/(Temp[t]+273.15)))
                  
                   # Growth in steps
                  for (i in 1:iter ) {
                    r =  TC* (v * (f/ L_b - 1/ Lm)/ (f + g)) # *Temp 1/d, specific growth rate
                    dL = r/iter * L/ 3
                    L<-L+dL
                    
                    #record instar durations
                    # time L1
                    if ((L >= L_2) & (L_1set == FALSE)){
                      res_times[m,1]<- t-1+ i/iter
                      t_lapse <- t-1+ i/iter
                      L_1set = TRUE
                    }
                    # time L2
                    if ((L >= L_3) & (L_2set == FALSE)){
                      res_times[m,2] <- t-1+ i/iter - t_lapse
                      t_lapse <- t-1+ i/iter
                      L_2set = TRUE
                    }
                    # time L3
                    if ((L >= L_4) & (L_3set == FALSE)){
                      res_times[m,3] <- t-1+ i/iter - t_lapse
                      t_lapse <- t-1+ i/iter
                      L_3set = TRUE
                    }
                    # time L4
                    if ((L >= L_5) & (L_4set == FALSE)){
                      res_times[m,4] <- t-1+ i/iter - t_lapse
                      t_lapse <- t-1+ i/iter
                      L_4set = TRUE
                    }
                    # time L5
                    if ((L >= L_6) & (L_5set == FALSE)){
                      res_times[m,5] <- t-1+ i/iter - t_lapse
                      t_lapse <- t-1+ i/iter
                      L_5set = TRUE
                    }
                    # time L6
                    if ((L >= L_e) & (L_6set == FALSE)){
                      res_times[m,6] <- t-1+ i/iter - t_lapse
                      #res_times[m,7] <- t-1+ i/iter 
                      t_lapse <- t-1+ i/iter
                      L_6set = TRUE
                    }
                    
                  }

        }  #end time
        
}#end MC     

  t_L1 <- sort(res_times[,1])
  t_L2 <- sort(res_times[,2])
  t_L3 <- sort(res_times[,3])
  t_L4 <- sort(res_times[,4])
  t_L5 <- sort(res_times[,5])
  t_L6 <- sort(res_times[,6])
  #t_L  <- sort(res_times[,7])

#create plots ---------------------------------------------------------
par(mfrow = c(3, 2))
int<-0.2
t_min<- min(res_times)-1.2*int
t_max<- max(res_times)

 # t_min<-2
 # t_max<- 8

font.scale<-1.5


hist(t_L1, main = "L1", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
#curve(dnorm(x, mean=mean(t_L1), sd=sd(t_L1)), add=TRUE, col = "blue" , lwd=2) 
  abline(v=mean(t_L1), col="red", lwd=3)
  
hist(t_L2, main = "L2", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  abline(v=mean(t_L2), col="red", lwd=3)
  
hist(t_L3, main = "L3", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  abline(v=mean(t_L3), col="red", lwd=3)
  
hist(t_L4, main = "L4", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  abline(v=mean(t_L4), col="red", lwd=3) 

hist(t_L5, main = "L5", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  abline(v=mean(t_L5), col="red", lwd=3)

hist(t_L6, main = "L6", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, xlim=c(t_min, t_max), col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  abline(v=mean(t_L6), col="red", lwd=3)
  
  # t_min <- min(t_L)-1.2*int
  # hist(t_L, main = "total L", xlab = "Development time [d]", cex.main=font.scale, cex.axis=font.scale, cex.lab=font.scale, col = "#99CCFF", breaks=seq(t_min,ceiling(t_max),by=int))
  # abline(v=mean(t_L), col="red", lwd=3)  

#mean(t_L1)+mean(t_L2)
  # mean(t_L)
  # sd(t_L)
  
 # min(t_L)
 # max(t_L)
  mean(t_L1)+ mean(t_L2)+ mean(t_L3)+mean(t_L5)+mean(t_L6)
